wearable = {}

function wearable.prerequisitesPresent(specializations)
  return true
end

function wearable:preLoad(savegame)
  self.setWearAmount = wearable.setWearAmount
  self.getWearAmount = wearable.getWearAmount
  self.getWearMultiplier = wearable.getWearMultiplier
  self.addWearableNode = wearable.addWearableNode
  self.removeWearableNode = wearable.removeWearableNode
end

function wearable:load(savegame)
  self.sentWearAmount = 0
  self.wearAmount = 0
  if hasXMLProperty(self.xmlFile, "vehicle.wearable#wearDuration") then
    self.wearableNodes = {}
    self.wearDuration = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.wearable#wearDuration"), 1) * 60 * 1000
    if self.wearDuration ~= 0 then
      self.wearDuration = 1 / self.wearDuration
    end
    self.fieldWearMultiplier = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.wearable#fieldMultiplier"), 2)
  end
end

function wearable:postLoad(savegame)
  if self.wearableNodes ~= nil then
    for _, comp in pairs(self.components) do
      self:addWearableNode(comp.node)
    end
    if savegame ~= nil then
      self:setWearAmount(Utils.getNoNil(getXMLFloat(savegame.xmlFile, savegame.key .. "#wearAmount"), 0), true)
    else
      self:setWearAmount(0, true)
    end
  end
end

function wearable:readStream(streamId, connection)
  if self.wearableNodes ~= nil then
    local wearAmount = streamReadInt8(streamId) / 100
    self.sentWearAmount = wearAmount
    self:setWearAmount(wearAmount, true)
  end
end

function wearable:writeStream(streamId, connection)
  if self.wearableNodes ~= nil then
    streamWriteInt8(streamId, math.floor(self:getWearAmount() * 100))
  end
end

function wearable:readUpdateStream(streamId, timestamp, connection)
  if connection:getIsServer() and self.wearableNodes ~= nil then
    local weary = streamReadBool(streamId)
    if weary then
      local wearAmount = streamReadInt8(streamId) / 100
      self:setWearAmount(wearAmount, true)
    end
  end
end

function wearable:writeUpdateStream(streamId, connection, dirtyMask)
  if not connection:getIsServer() and self.wearableNodes ~= nil then
    streamWriteInt8(streamId, math.floor(self.sentWearAmount * 100))
  end
end

function wearable:delete()
end

function wearable:mouseEvent(posX, posY, isDown, isUp, button)
end

function wearable:keyEvent(unicode, sym, modifier, isDown)
end

function wearable:update(dt)
end

function wearable:updateTick(dt)
  if self.wearableNodes ~= nil then
    if self.isServer then
      if g_currentMission.environment.lastRainScale > 0.1 and g_currentMission.environment.timeSinceLastRain < 30 then
        local amount = self:getWearAmount()
        if amount > 0.5 then
          amount = self:getWearAmount() + (dt / self.wearDuration)
        end
        self:setWearAmount(amount)
      else
        if self:getIsActive() or self.isActive then
          self:setWearAmount(self:getWearAmount() + (dt * self.wearDuration)*self:getWearMultiplier())
        end
      end
    end
  end
end

function wearable:draw()
end

function wearable:setWearAmount(wearAmount, force)
  if self.wearableNodes ~= nil then
    self.wearAmount = Utils.clamp(Utils.getNoNil(wearAmount, self.wearAmount), 0, 1)
    if math.abs(self.sentWearAmount - self.wearAmount) > 0.01 or force then
      for _, node in pairs(self.wearableNodes) do
        local _, y, z, w = getShaderParameter(node, "RDT")
        setShaderParameter(node, "RDT", self.wearAmount, y, z, w, false)
      end
      if self.isServer then
        self.sentWearAmount = self.wearAmount
      end
    end
  end
end

function wearable:addWearableNode(node)
  if self.wearableNodes ~= nil and node ~= nil then
    Utils.getNodesByShaderParam(node, "RDT", self.wearableNodes)
  end
end

function wearable:removeWearableNode(node)
  if self.wearableNodes ~= nil and node ~= nil then
    self.wearableNodes[node] = nil
  end
end

function wearable:getWearAmount(superFunc)
  return self.wearAmount
end

function wearable:getWearMultiplier(superFunc)
  local multiplier = 1
  if self:getLastSpeed() < 1 then
    multiplier = 0
  end
  if self:getIsOnField() then
    multiplier = multiplier * self.fieldWearMultiplier
  end
  if superFunc ~= nil then
    multiplier = multiplier + superFunc(self)
  end
  return multiplier
end

function wearable:getSaveAttributesAndNodes(nodeIdent)
  local attributes = 'wearAmount="' .. tostring(self.wearAmount) .. '"'
  return attributes, nil
end
